@@ -39,10 +39,9 @@ static GtkCheckMenuItem *cp_view_large_icons;
 
 struct _AppData
 {
-    HildonApp *app;
-    HildonAppView *appview;
+    HildonProgram *program;
+    HildonWindow *window;
     GtkWidget * grid;
-    GtkWindow *window;
     osso_context_t *osso;
 
     /* for state save data */
@@ -59,21 +58,9 @@ AppData state_data;
 /* delay to wait from callback to actually reading the entries in msecs*/
 #define DIRREADDELAY 500
 
-/* Message bus definitions */
-/* Definitions for password validation call */
-#define HILDON_CP_DBUS_MCE_SERVICE "com.nokia.mce"
-#define HILDON_CP_DBUS_MCE_REQUEST_IF "com.nokia.mce.request"
-#define HILDON_CP_DBUS_MCE_REQUEST_PATH "/com/nokia/mce/request"
-#define HILDON_CP_MCE_PASSWORD_VALIDATE "validate_devicelock_code"
-#define HILDON_CP_DEFAULT_SALT "$1$JE5Gswee$"
+#define HILDON_CP_RFS_SCRIPT "/usr/sbin/osso-app-killer-rfs.sh"
 
-/* Definitions for appkiller call */
-#define HILDON_CP_APPL_DBUS_NAME "osso_app_killer"
-#define HILDON_CP_SVC_NAME "com.nokia." HILDON_CP_APPL_DBUS_NAME
-#define HILDON_CP_RFS_SHUTDOWN_OP "/com/nokia/" \
- HILDON_CP_APPL_DBUS_NAME "/rfs_shutdown"
-#define HILDON_CP_RFS_SHUTDOWN_IF HILDON_CP_SVC_NAME ".rfs_shutdown"
-#define HILDON_CP_RFS_SHUTDOWN_MSG "shutdown"
+#define HILDON_CP_CUD_SCRIPT "/usr/sbin/osso-app-killer-cud.sh"
 
 /* Definitions for mobile operator wizard launch*/
 #define HILDON_CONTROL_PANEL_DBUS_OPERATOR_WIZARD_SERVICE "operator_wizard"
@@ -92,27 +79,6 @@ static void _init_gui (void);
 static void _large_icons(void);
 static void _small_icons(void);
 
-/* Make a DBUS system call  */
-static osso_return_t _cp_run_rpc_system_request (osso_context_t *osso, 
-                                                 const gchar *service, 
-                                                 const gchar *object_path,
-                                                 const gchar *interface, 
-                                                 const gchar *method,
-                                                 osso_rpc_t *retval, 
-                                                 int first_arg_type,
-                                                 ...);
-
-static osso_return_t _rpc_system_request (osso_context_t *osso, 
-                                          DBusConnection *conn,
-                                          const gchar *service, 
-                                          const gchar *object_path,
-                                          const gchar *interface, 
-                                          const gchar *method,
-                                          osso_rpc_t *retval, 
-                                          int first_arg_type,
-                                          va_list var_args);
-static void _append_args(DBusMessage *msg, int type, va_list var_args);
-static void _get_arg(DBusMessageIter *iter, osso_rpc_t *retval);
 
 /* Create & free state data */
 static void _create_data(void);
@@ -131,11 +97,12 @@ static gint _cp_keyboard_listener(GtkWidget *widget,
                                   GdkEventKey * keyevent, gpointer data);
 
 /* Callback for topping */
-static void _topmost_status_acquire(void);
-/* Callback for saving state */
-static void _topmost_status_lose(void);
+static void _topmost_status_change(GObject *gobject, GParamSpec *arg1,
+                                   gpointer user_data);;
 /* Callback for resetting factory settings */
 static gboolean _reset_factory_settings( GtkWidget *widget, gpointer data );
+/* Callback for clear user data */
+static gboolean _clear_user_data( GtkWidget *widget, gpointer data );
 /* Callback for operator wizard launch */
 static void _run_operator_wizard( GtkWidget *widget, gpointer data );
 /* Callback for help function */
@@ -160,7 +127,6 @@ static void _hw_signal_cb( osso_hw_state_t *state, gpointer data );
 int 
 main(int argc, char ** argv)
 {
-
     setlocale( LC_ALL, "" );
     
     bindtextdomain( PACKAGE, LOCALEDIR );
@@ -168,6 +134,9 @@ main(int argc, char ** argv)
     bind_textdomain_codeset( PACKAGE, "UTF-8" );
 
     textdomain( PACKAGE );
+
+    /* Don't display the application name in the title bar */
+    g_set_application_name ("");
      
     _init_dnotify( CONTROLPANEL_ENTRY_DIR, 
                    g_getenv(ADDITIONAL_CP_APPLETS_DIR_ENVIRONMENT) == NULL ?
@@ -189,7 +158,7 @@ main(int argc, char ** argv)
 
     _enforce_state();     /* realize the saved state */
 
-    gtk_widget_show_all( GTK_WIDGET( state_data.app ) );
+    gtk_widget_show_all( GTK_WIDGET( state_data.window ) );
 
     if(state_data.execute==1) {
         _launch(hildon_cp_applist_get_entry(state_data.focused_filename),
@@ -285,28 +254,24 @@ static void _init_gui(void)
     GSList *menugroup = NULL;
     GtkWidget *grid = NULL;
 
-    state_data.app = HILDON_APP( hildon_app_new() );
+    state_data.window = HILDON_WINDOW( hildon_window_new() );
+    state_data.program = HILDON_PROGRAM( hildon_program_get_instance() );
 
-    hildon_app_set_title( state_data.app, HILDON_CONTROL_PANEL_TITLE );
+    hildon_program_add_window( state_data.program, state_data.window );
 
-    hildon_app_set_two_part_title( state_data.app, FALSE );
-
-    g_signal_connect( G_OBJECT( state_data.app ), "destroy", 
+    gtk_window_set_title( GTK_WINDOW( state_data.window ),
+            HILDON_CONTROL_PANEL_TITLE );
+    
+    g_signal_connect( G_OBJECT( state_data.window ), "destroy", 
                       G_CALLBACK( _my_quit ),NULL);
-
-    state_data.appview = HILDON_APPVIEW( hildon_appview_new(
-                                             "MainView" ) );
-
-    hildon_app_set_appview( state_data.app, state_data.appview );
     
-    g_signal_connect(G_OBJECT(state_data.app), "topmost_status_lose",
-                     G_CALLBACK(_topmost_status_lose), NULL);
-    g_signal_connect(G_OBJECT(state_data.app), "topmost_status_acquire",
-                     G_CALLBACK(_topmost_status_acquire), NULL);
-    
-    /* Menu */
-    menu = GTK_MENU( hildon_appview_get_menu( state_data.appview ) );
+    g_signal_connect(G_OBJECT(state_data.program), "notify::is-topmost",
+                     G_CALLBACK(_topmost_status_change), NULL);
 
+    menu = GTK_MENU( gtk_menu_new() );
+
+    hildon_window_set_menu( state_data.window, menu );
+    
     mi = gtk_menu_item_new_with_label( HILDON_CONTROL_PANEL_MENU_OPEN );
     
     g_signal_connect( G_OBJECT( mi ), "activate", 
@@ -380,12 +345,21 @@ static void _init_gui(void)
 
     /* Reset Factory Settings */
     mi = gtk_menu_item_new_with_label
-        ( HILDON_CONTROL_PANEL_MENU_RESET_SETTINGS);
+        ( HILDON_CONTROL_PANEL_MENU_RFS );
 
     gtk_menu_shell_append( GTK_MENU_SHELL( sub_tools ), mi );
     
     g_signal_connect( G_OBJECT( mi ), "activate", 
                       G_CALLBACK(_reset_factory_settings), NULL);
+    
+    /* Clean User Data */
+    mi = gtk_menu_item_new_with_label
+        ( HILDON_CONTROL_PANEL_MENU_CUD );
+
+    gtk_menu_shell_append( GTK_MENU_SHELL( sub_tools ), mi );
+    
+    g_signal_connect( G_OBJECT( mi ), "activate", 
+                      G_CALLBACK(_clear_user_data), NULL);
 
     
     /* Help! */
@@ -411,7 +385,7 @@ static void _init_gui(void)
     
     hildon_cp_applist_initialize( _launch, NULL,
                                     _focus_change, NULL,
-                                    GTK_WIDGET( state_data.appview ), 
+                                    GTK_WIDGET( state_data.window ), 
                                     CONTROLPANEL_ENTRY_DIR);
 
     grid = hildon_cp_applist_get_grid();
@@ -423,10 +397,7 @@ static void _init_gui(void)
 	g_value_unset(&val);
     }
 
-    gtk_widget_show_all(GTK_WIDGET(state_data.app));
-
-    state_data.window = GTK_WINDOW( gtk_widget_get_toplevel(
-                                        GTK_WIDGET( state_data.app ) ) );
+    gtk_widget_show_all(GTK_WIDGET(state_data.window));
 
     state_data.grid = hildon_cp_applist_get_grid();
 
@@ -489,6 +460,7 @@ static void _save_state( gboolean clear_state ) {
                                                         state_data.grid));
     gchar contents[HILDON_CONTROL_PANEL_STATEFILE_MAX];
     gint fd, ret;
+					
 
     if (clear_state) {
         ret=g_sprintf(contents, "%s\n",
@@ -498,11 +470,19 @@ static void _save_state( gboolean clear_state ) {
     {
         if (state_data.focused_filename != NULL) 
 	  {
-  	      osso_cp_plugin_save_state(
-		 		        state_data.osso,
-					state_data.focused_filename,
+          
+          MBDotDesktop* dd = 
+              hildon_cp_applist_get_entry( state_data.focused_filename );
+          char * lib_file = NULL;
+          if(dd && (lib_file = 
+               (char *)mb_dotdesktop_get(dd, "X-control-panel-plugin")))
+          {
+              osso_cp_plugin_save_state(
+		 		    state_data.osso,
+					lib_file,
 					NULL
 					);
+          }
 	  }
         ret = g_snprintf(contents, HILDON_CONTROL_PANEL_STATEFILE_MAX,
                          "focused=%s\n"
@@ -540,7 +520,7 @@ static void _save_state( gboolean clear_state ) {
 
 static void _retrieve_state( void ) {
     gchar buff[HILDON_CONTROL_PANEL_STATEFILE_MAX];
-    size_t rc;
+    ssize_t rc;
     gint fd;
     gchar *eq;
     gchar *nl;
@@ -548,7 +528,7 @@ static void _retrieve_state( void ) {
     gint length;
 
     fd = osso_state_open_read(state_data.osso);
-    if( fd == -1)
+    if(fd == -1)
     {
         return;
     }
@@ -561,7 +541,13 @@ static void _retrieve_state( void ) {
         osso_log(LOG_ERR,
                  "Error retrieving state -- too long state file %d", rc);
         return;
-    }   
+    }
+    if(rc == -1)
+    {
+        osso_log(LOG_ERR,
+                 "Error retrieving state -- error on reading the state file");
+        return;
+    }
    
     start = buff;
     length = rc;
@@ -723,6 +709,7 @@ static void _retrieve_configuration()
         {
             osso_log(LOG_ERR, 
                      "Couldn't load statusinfo from configure file");
+            state_data.icon_size = 0;
         }
         else {
             g_return_if_fail (iconsize_data != NULL);
@@ -736,7 +723,7 @@ static void _retrieve_configuration()
                 state_data.icon_size = 0;
             }
             else {
-                state_data.icon_size = 1;
+                state_data.icon_size = 0;
             }
         }
     }
@@ -791,23 +778,23 @@ static gint _cp_keyboard_listener( GtkWidget * widget,
 
 /***** Top/Untop callbacks *****/
 
-/* save state when we get untopped*/
-static void _topmost_status_lose(void)
+static void _topmost_status_change(GObject *gobject, GParamSpec *arg1,
+                                   gpointer user_data)
 {
-    _save_state(FALSE);
-    hildon_app_set_killable(state_data.app, TRUE);
+    HildonProgram *program = HILDON_PROGRAM( gobject );
+
+    if( hildon_program_get_is_topmost (program) )
+    {
+        hildon_program_set_can_hibernate(program, FALSE);
+    }
+    else
+    {
+        _save_state(FALSE);
+        hildon_program_set_can_hibernate(program, TRUE);
+    }
+
 }
 
-/* Remove the "save to OOM kill" hint when topped */
-
-static void _topmost_status_acquire(void)
-{
-    mb_util_window_activate(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(
-                                GTK_WIDGET(state_data.app)->window));
-    hildon_app_set_killable(state_data.app, FALSE);
-}
-
-
 /****** Menu callbacks ******/
 
 static void _my_open( GtkWidget *widget, gpointer data )
@@ -818,7 +805,6 @@ static void _my_open( GtkWidget *widget, gpointer data )
 
 static void _my_quit(GtkWidget *widget, gpointer data)
 {
-    hildon_app_set_killable(state_data.app, FALSE);
     _save_state( TRUE );
     gtk_main_quit();
 }
@@ -885,204 +871,21 @@ static void _run_operator_wizard( GtkWidget *widget, gpointer data)
             osso_log(LOG_ERR, "Unknown error type %d", returnstatus);
     }
     
+    osso_rpc_free_val (&returnvalues);
 }
 
 static gboolean _reset_factory_settings( GtkWidget *widget, gpointer data )
 {
 
-    gint i, retry;
-    gchar *password;
-    gchar *crypted_password;
-    GtkWidget *confirm_dialog;
-    gchar *dollarsign = "$";
-    osso_rpc_t returnvalues;
-    osso_return_t returnstatus;
+    hildon_cp_rfs( state_data.osso, HILDON_CP_RFS_WARNING, HILDON_CP_RFS_WARNING_TITLE, HILDON_CP_RFS_SCRIPT, HILDON_CP_RFS_HELP_TOPIC );
 
-    retry=1;
+    return TRUE;
+}
 
-    if ( data == NULL ) { /* In retries, we don't confirm again  */
-        
-        confirm_dialog = hildon_note_new_confirmation
-            (GTK_WINDOW(state_data.window), 
-             RESET_FACTORY_SETTINGS_INFOBANNER);
-        
-        hildon_note_set_button_texts
-            (HILDON_NOTE(confirm_dialog),
-             RESET_FACTORY_SETTINGS_INFOBANNER_CLOSE_ALL,
-             RESET_FACTORY_SETTINGS_INFOBANNER_CANCEL);
-        
-        gtk_widget_show_all(confirm_dialog);
-        i = gtk_dialog_run(GTK_DIALOG(confirm_dialog));
+static gboolean _clear_user_data( GtkWidget *widget, gpointer data )
+{
 
-        gtk_widget_destroy(GTK_WIDGET(confirm_dialog));
-
-        if (i == GTK_RESPONSE_CANCEL ||
-            i == GTK_RESPONSE_DELETE_EVENT) {
-            return TRUE;    
-        }
-
-    }
-    else {
-        retry = GPOINTER_TO_INT(data) +1;
-    }
-    
-    GtkWidget *pw_dialog = (hildon_get_password_dialog_new(
-                                GTK_WINDOW(state_data.window), FALSE));
-
-    hildon_get_password_dialog_set_title
-        (HILDON_GET_PASSWORD_DIALOG(pw_dialog), 
-         RESET_FACTORY_SETTINGS_PASSWORD_DIALOG_TITLE);
-
-    hildon_get_password_dialog_set_caption
-        (HILDON_GET_PASSWORD_DIALOG(pw_dialog), 
-         RESET_FACTORY_SETTINGS_PASSWORD_DIALOG_CAPTION);
-/*    
-    hildon_get_password_dialog_set_max_characters
-        (HILDON_GET_PASSWORD_DIALOG(pw_dialog),
-         HILDON_CP_ASSUMED_LOCKCODE_MAXLENGTH);
-
-         open issue: widget spec just dropped max characters altogether.
-         retaining code, but as the logical string no longer exits...
-*/
-    g_object_set( G_OBJECT(pw_dialog),
-                  "numbers_only", NULL, NULL );
-    
-
-    gtk_widget_show (pw_dialog);
-    
-    switch(gtk_dialog_run (GTK_DIALOG(pw_dialog)))
-    {
-      case GTK_RESPONSE_CANCEL:
-      case GTK_RESPONSE_DELETE_EVENT:
-
-        if (pw_dialog !=NULL)
-            gtk_widget_destroy(GTK_WIDGET(pw_dialog));        
-        return TRUE;
-    }  
-
-    password = strdup(hildon_get_password_dialog_get_password
-                      (HILDON_GET_PASSWORD_DIALOG(pw_dialog)));
-    
-    if (password == NULL || strcmp(password,"") == 0) {        
-
-        gtk_infoprint (GTK_WINDOW(state_data.window), 
-                       RESET_FACTORY_SETTINGS_IB_NO_LOCKCODE );
-        if (pw_dialog)
-        {            
-            gtk_widget_destroy(GTK_WIDGET(pw_dialog));
-        }
-        if (password != NULL) 
-        {
-            g_free (password);
-        }
-        _reset_factory_settings(widget, GINT_TO_POINTER(retry));
-        return TRUE;
-    }
-
-    crypted_password = crypt(password, HILDON_CP_DEFAULT_SALT);
-    g_free(password);
-
-    gint c = (gint)dollarsign[0];
-
-    password = rindex (crypted_password, c)+1;
-
-    returnstatus = _cp_run_rpc_system_request
-        (state_data.osso, 
-         HILDON_CP_DBUS_MCE_SERVICE, 
-         HILDON_CP_DBUS_MCE_REQUEST_PATH,
-         HILDON_CP_DBUS_MCE_REQUEST_IF,
-         HILDON_CP_MCE_PASSWORD_VALIDATE,
-         &returnvalues, 
-         DBUS_TYPE_STRING,
-         password,
-         DBUS_TYPE_STRING,
-         HILDON_CP_DEFAULT_SALT,
-         DBUS_TYPE_INVALID);
-     
-    password = NULL; /* password pointed to middle of crypted password */
-
-    switch (returnstatus)
-    {
-    case OSSO_INVALID:
-        osso_log(LOG_ERR, "Invalid parameter\n");
-        fprintf(stderr, "got invalid");
-        break;
-    case OSSO_RPC_ERROR:
-    case OSSO_ERROR:
-    case OSSO_ERROR_NAME:
-    case OSSO_ERROR_NO_STATE:
-    case OSSO_ERROR_STATE_SIZE:
-        if (returnvalues.type == DBUS_TYPE_STRING) {
-            osso_log(LOG_ERR, "Lockcode query call failed: %s\n", 
-                     returnvalues.value.s);
-        }
-        else {
-            osso_log(LOG_ERR, 
-                     "Lockcode query call failed, unspecified");
-        }
-        break;
-        
-    case OSSO_OK:
-        break;
-    default:
-        osso_log(LOG_ERR, "Unknown error type %d", returnstatus);
-    }
-    
-    
-    if (returnvalues.type == DBUS_TYPE_BOOLEAN &&
-        returnvalues.value.b == TRUE) { 
-        
-        returnstatus = osso_rpc_run(state_data.osso, 
-                                    HILDON_CP_SVC_NAME, 
-                                    HILDON_CP_RFS_SHUTDOWN_OP,
-                                    HILDON_CP_RFS_SHUTDOWN_IF, 
-                                    HILDON_CP_RFS_SHUTDOWN_MSG,
-                                    &returnvalues, 
-                                    DBUS_TYPE_INVALID);
-        
-        
-
-        switch (returnstatus)
-        {
-        case OSSO_INVALID:
-            osso_log(LOG_ERR, "Invalid parameter\n");
-            break;
-        case OSSO_RPC_ERROR:
-        case OSSO_ERROR:
-        case OSSO_ERROR_NAME:
-        case OSSO_ERROR_NO_STATE:
-        case OSSO_ERROR_STATE_SIZE:
-            if (returnvalues.type == DBUS_TYPE_STRING) {
-                osso_log(LOG_ERR, "Appkiller call failed: %s\n", 
-                         returnvalues.value.s);
-            }
-            else {
-                osso_log(LOG_ERR, 
-                         "Appkiller call failed, unspecified");
-            }
-            break;
-            
-        case OSSO_OK:
-            return TRUE;
-            /* There is no real need for case OSSO_OK: we'll be dead
-               ... if everyhting goes as planned, anyway! */
-        default:
-            osso_log(LOG_ERR, "Unknown error type %d", returnstatus);
-        }
-
-    }
-    else if (returnvalues.type == DBUS_TYPE_BOOLEAN &&
-             returnvalues.value.b == FALSE) { 
-        
-        gtk_infoprint (GTK_WINDOW(state_data.window), 
-                       RESET_FACTORY_SETTINGS_IB_WRONG_LOCKCODE );
-        gtk_widget_destroy(GTK_WIDGET(pw_dialog));
-        _reset_factory_settings(widget, GINT_TO_POINTER(retry));
-        return TRUE;
-    }
-
-    if (pw_dialog)
-        gtk_widget_destroy(GTK_WIDGET(pw_dialog));
+    hildon_cp_rfs( state_data.osso, HILDON_CP_CUD_WARNING, HILDON_CP_CUD_WARNING_TITLE, HILDON_CP_CUD_SCRIPT, HILDON_CP_CUD_HELP_TOPIC );
 
     return TRUE;
 }
@@ -1142,15 +945,17 @@ static void _hw_signal_cb( osso_hw_state_t *state, gpointer data )
 static void _launch(MBDotDesktop* dd, gpointer data, gboolean user_activated )
 {
     int ret;
+    char * lib_file = NULL;
+
+    if(dd && (lib_file = 
+                (char *)mb_dotdesktop_get(dd, "X-control-panel-plugin")))
+    {
 
-    if(dd) {
         state_data.execute=1;
-        /* FIXME: What if mb_dotdesktop_get returns NULL? --> Segfault */
         ret = osso_cp_plugin_execute( state_data.osso, 
-                                      (char *)mb_dotdesktop_get(
-                                          dd, "X-control-panel-plugin"),
+                                      lib_file,
                                       (gpointer)state_data.window,
-				      user_activated );
+                                      user_activated );
         state_data.execute = 0;
     }
 }
@@ -1165,197 +970,3 @@ static void _focus_change(MBDotDesktop* dd, gpointer data )
             mb_dotdesktop_get_filename(dd));
     }
 }
-
-/***** DBUS system calls *****/
-
-
-
-static osso_return_t _cp_run_rpc_system_request (osso_context_t *osso, 
-                                                 const gchar *service, 
-                                                 const gchar *object_path,
-                                                 const gchar *interface, 
-                                                 const gchar *method,
-                                                 osso_rpc_t *retval, 
-                                                 int first_arg_type, ...)
-{    
-    osso_return_t ret;
-    va_list arg_list;
-    
-    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
-	return OSSO_INVALID;
-    
-    va_start(arg_list, first_arg_type);
-    
-    ret = _rpc_system_request(osso, 
-                              (DBusConnection*)osso_get_sys_dbus_connection
-                              (osso), 
-                              service, 
-                              object_path, interface,
-                              method, retval, first_arg_type, arg_list);
-    va_end(arg_list);
-    return ret;
-}
-
-static osso_return_t _rpc_system_request (osso_context_t *osso, 
-                                          DBusConnection *conn,
-                                          const gchar *service, 
-                                          const gchar *object_path,
-                                          const gchar *interface, 
-                                          const gchar *method,
-                                          osso_rpc_t *retval, 
-                                          int first_arg_type,
-                                          va_list var_args)
-{
-    gint timeout;
-    DBusMessage *msg;
-    DBusError err;
-    DBusMessage *ret;
-    osso_return_t timeout_retval;
-
-    if(conn == NULL)
-	return OSSO_INVALID;
-    
-    dprint("New method: %s%s::%s:%s",service,object_path,interface,method);
-    msg = dbus_message_new_method_call(service, object_path,
-				       interface, method);
-    if(msg == NULL)
-    {
-	return OSSO_ERROR;
-    }
-
-
-    if(first_arg_type != DBUS_TYPE_INVALID)
-	_append_args(msg, first_arg_type, var_args);
-    
-    dbus_message_set_auto_activation(msg, FALSE);
-
-    dbus_error_init(&err);
-    timeout_retval = osso_rpc_get_timeout(osso, &timeout);
-    ret = dbus_connection_send_with_reply_and_block(conn, msg, 
-                                                    timeout,
-                                                    &err);
-    if (!ret) {
-        dprint("Error return: %s",err.message);
-        retval->type = DBUS_TYPE_STRING;
-        retval->value.s = g_strdup(err.message);
-        dbus_error_free(&err);
-        return OSSO_RPC_ERROR;
-    }
-    else {
-        DBusMessageIter iter;
-        DBusError err;
-
-        switch(dbus_message_get_type(ret)) {
-        case DBUS_MESSAGE_TYPE_ERROR:
-            dbus_set_error_from_message(&err, ret);
-            retval->type = DBUS_TYPE_STRING;
-            retval->value.s = g_strdup(err.message);
-            dbus_error_free(&err);
-            return OSSO_RPC_ERROR;
-        case DBUS_MESSAGE_TYPE_METHOD_RETURN:
-            dprint("message return");
-            dbus_message_iter_init(ret, &iter);
-	    
-            _get_arg(&iter, retval);
-            
-            dbus_message_unref(ret);
-            
-            return OSSO_OK;
-        default:
-            retval->type = DBUS_TYPE_STRING;
-            retval->value.s = g_strdup("Invalid return value");
-            return OSSO_RPC_ERROR;
-        }
-    }
-}    
-
-
-static void _append_args(DBusMessage *msg, int type, va_list var_args)
-{
-    while(type != DBUS_TYPE_INVALID) {
-	gchar *s;
-	gint32 i;
-	guint32 u;
-	gdouble d;
-	gboolean b;
-	
-	switch(type) {
-	  case DBUS_TYPE_INT32:
-	    i = va_arg(var_args, gint32);
-	    dbus_message_append_args(msg, type, i, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_UINT32:
-	    u = va_arg(var_args, guint32);
-	    dbus_message_append_args(msg, type, u, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_BOOLEAN:
-	    b = va_arg(var_args, gboolean);
-	    dbus_message_append_args(msg, type, b, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_DOUBLE:
-	    d = va_arg(var_args, gdouble);
-	    dbus_message_append_args(msg, type, d, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_STRING:
-	    s = va_arg(var_args, gchar *);
-	    if(s == NULL)
-		dbus_message_append_args(msg, DBUS_TYPE_NIL,
-					 DBUS_TYPE_INVALID);
-	    else
-		dbus_message_append_args(msg, type, s, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_NIL:	    
-		dbus_message_append_args(msg, DBUS_TYPE_NIL,
-					 DBUS_TYPE_INVALID);
-	    break;
-	  default:
-	    break;	    
-	}
-	
-	type = va_arg(var_args, int);
-    }
-    return;
-}
-
-static void _get_arg(DBusMessageIter *iter, osso_rpc_t *retval)
-{
-    dprint("");
-
-    retval->type = dbus_message_iter_get_arg_type(iter);
-    switch(retval->type) {
-      case DBUS_TYPE_INT32:
-	retval->value.i = dbus_message_iter_get_int32(iter);
-	dprint("got INT32:%d",retval->value.i);
-	break;
-      case DBUS_TYPE_UINT32:
-	retval->value.u = dbus_message_iter_get_uint32(iter);
-	dprint("got UINT32:%u",retval->value.u);
-	break;
-      case DBUS_TYPE_BOOLEAN:
-	retval->value.b = dbus_message_iter_get_boolean(iter);
-	dprint("got BOOLEAN:%s",retval->value.s?"TRUE":"FALSE");
-	break;
-      case DBUS_TYPE_DOUBLE:
-	retval->value.d = dbus_message_iter_get_double(iter);
-	dprint("got DOUBLE:%f",retval->value.d);
-	break;
-      case DBUS_TYPE_STRING:
-	retval->value.s = 
-	    g_strdup(dbus_message_iter_get_string(iter));
-	if(retval->value.s == NULL) {
-	    retval->type = DBUS_TYPE_INVALID;
-	}
-	dprint("got STRING:'%s'",retval->value.s);
-	break;
-      case DBUS_TYPE_NIL:
-	retval->value.s = NULL;	    
-	dprint("got NIL");
-	break;
-      default:
-	retval->type = DBUS_TYPE_INVALID;
-	retval->value.i = 0;	    
-	dprint("got unknown type:'%c'(%d)",retval->type,retval->type);
-	break;	
-    }
-}
