@@ -49,12 +49,13 @@
 #include <dbus/dbus.h>
 
 #include <hildon-widgets/hildon-get-password-dialog.h>
-#include <hildon-widgets/hildon-app.h>
+#include <hildon-widgets/hildon-window.h>
+#include <hildon-widgets/hildon-program.h>
 #include <hildon-widgets/hildon-note.h>
 #include <hildon-widgets/hildon-grid.h>
 #include <hildon-base-lib/hildon-base-dnotify.h>
-#include <hildon-lgpl/hildon-widgets/gtk-infoprint.h>
-#include <hildon-lgpl/hildon-widgets/hildon-defines.h>
+#include <hildon-widgets/gtk-infoprint.h>
+#include <hildon-widgets/hildon-defines.h>
 
 #include <libmb/mbdotdesktop.h>
 #include <libmb/mbutil.h>
@@ -80,42 +81,25 @@ G_BEGIN_DECLS
 #define HILDON_CONTROL_PANEL_MENU_LARGE_ITEMS _("copa_me_view_large")
 #define HILDON_CONTROL_PANEL_MENU_SUB_TOOLS _("copa_me_tools")
 #define HILDON_CONTROL_PANEL_MENU_SETUP_WIZARD _("copa_me_tools_setup_wizard")
-#define HILDON_CONTROL_PANEL_MENU_RESET_SETTINGS _("copa_me_tools_rfs")
+#define HILDON_CONTROL_PANEL_MENU_RFS _("copa_me_tools_rfs")
+#define HILDON_CONTROL_PANEL_MENU_CUD _("copa_me_tools_cud")
 #define HILDON_CONTROL_PANEL_MENU_HELP _("copa_me_tools_help")
 #define HILDON_CONTROL_PANEL_MENU_CLOSE _("copa_me_close")
 
-#define RESET_FACTORY_SETTINGS_PASSWORD_DIALOG_TITLE \
- _("rfs_ti_lock_code_protected")
-#define RESET_FACTORY_SETTINGS_PASSWORD_DIALOG_CAPTION _("rfs_fi_enter_lock_code")
-#define RESET_FACTORY_SETTINGS_PASSWORD_OK _("refs_bd_ok")
-#define RESET_FACTORY_SETTINGS_PASSWORD_CANCEL _("refs_bd_cancel2")
+#define RESET_FACTORY_SETTINGS_INFOBANNER_OK _("rfs_bd_ok")
+#define RESET_FACTORY_SETTINGS_INFOBANNER_CANCEL _("rfs_bd_cancel")
+#define HILDON_CP_RFS_WARNING _("refs_ia_text")
+#define HILDON_CP_CUD_WARNING _("cud_ia_text")
 
+#define HILDON_CP_RFS_WARNING_TITLE _("rfs_ti_restore")
+#define HILDON_CP_CUD_WARNING_TITLE _("cud_ti_clear")
 
-#define RESET_FACTORY_SETTINGS_INFOBANNER \
- _("refs_la_all_application_must_be_closed")
-#define RESET_FACTORY_SETTINGS_INFOBANNER_CLOSE_ALL _("refs_bd_close_all")
-#define RESET_FACTORY_SETTINGS_INFOBANNER_CANCEL _("refs_bd_cancel1")
+#define HILDON_CP_RFS_HELP_TOPIC "Features_restorefactorysettings_closealldialog"
+#define HILDON_CP_CUD_HELP_TOPIC "Features_clearuserdata_dialog"
 
-#define RESET_FACTORY_SETTINGS_IB_WRONG_LOCKCODE _("rfs_ia_incorrect_lock_code")
-#define RESET_FACTORY_SETTINGS_IB_NO_LOCKCODE _("rfs_ia_enter_lock_code")
+#define HILDON_CP_CODE_DIALOG_HELP_TOPIC "Features_restorefactorysettings_passwordquerydialog"
 
-/* The logical IDs for the applets, needed for automatical generation
-   of the .pot files*/
-
-#define HILDON_CP_APPINST_APPLET _("ai_ap_application_title")
-#define HILDON_CP_BUPREST_APPLET _("back_ti_dia001_title")
-#define HILDON_CP_CERTMAN_APPLET _("cema_ap_application_title")
-#define HILDON_CP_CONNE_APPLET _("conn_set_ti_conn_set")
-#define HILDON_CP_DATETIME_APPLET _("dati_ap_application_title")
-#define HILDON_CP_DEVICE_APPLET _("devi_ap_application_title")
-#define HILDON_CP_DISPLAY_APPLET _("disp_ap_application_title")
-#define HILDON_CP_LANGREG_APPLET _("cpal_ti_language_and_regional_title")
-#define HILDON_CP_MEMORY_APPLET _("memo_ti_memory")
-#define HILDON_CP_PERS_APPLET _("pers_ti_personalization")
-#define HILDON_CP_SCREENCAL_APPLET _("ctrp_ti_screen_calibration")
-#define HILDON_CP_SECURITY_APPLET _("secu_security_dialog_title")
-#define HILDON_CP_SOUNDS_APPLET _("snds_ti_sound_settings")
-#define HILDON_CP_TEXTINPUT_APPLET _("tein_ti_text_input_title")
+#define RESET_FACTORY_SETTINGS_IB_WRONG_LOCKCODE dgettext("hildon-libs", "secu_info_incorrectcode")
 
 #define HILDON_CP_SYSTEM_DIR ".osso/hildon-cp"
 #define HILDON_CP_CONF_USER_FILENAME "hildon-cp.conf"
@@ -125,12 +109,25 @@ G_BEGIN_DECLS
  "iconsize=%20s\n"
 #define HILDON_CP_MAX_FILE_LENGTH 30 /* CONF FILE FORMAT MUST NOT BE LARGER */
 
+
+#define HILDON_CP_DBUS_MCE_SERVICE "com.nokia.mce"
+#define HILDON_CP_DBUS_MCE_REQUEST_IF "com.nokia.mce.request"
+#define HILDON_CP_DBUS_MCE_REQUEST_PATH "/com/nokia/mce/request"
+#define HILDON_CP_MCE_PASSWORD_VALIDATE "validate_devicelock_code"
+#define HILDON_CP_DEFAULT_SALT "$1$JE5Gswee$"
+
 /* max length of state save file */
 
 #define HILDON_CONTROL_PANEL_STATEFILE_MAX 1024
 #define OSSO_HELP_ID_CONTROL_PANEL "Utilities_controlpanel_mainview"
 #define HILDON_CP_ASSUMED_LOCKCODE_MAXLENGTH 5 /* Note that max number of characters is removed from widget specs. */
 
+gboolean hildon_cp_rfs( osso_context_t *osso, 
+                        const char *warning,
+                        const char *title,
+                        const char *script,
+                        const char *help_topic );
+
 G_END_DECLS
 
 #endif
